@@ -296,7 +296,7 @@ setMethod("crossprod", signature(x = "matrix", y = "dgeMatrix"),
 
 setMethod("crossprod", signature(x = "numLike", y = "dgeMatrix"),
 	  function(x, y=NULL, boolArith=NA, ...)
-	      crossprod(as.matrix(as.double(x)), y, boolArith=boolArith, ...))
+	      crossprod(cbind(as.double(x), deparse.level=0L), y, boolArith=boolArith, ...))
 
 for(c.x in paste0(c("d", "l", "n"), "denseMatrix")) {
     setMethod("crossprod", signature(x = c.x, y = "missing"),
@@ -342,6 +342,14 @@ setMethod("crossprod", signature(x = "dtrMatrix", y = "matrix"),
 	      else
 		  .Call(dtrMatrix_matrix_mm, x, y, FALSE, TRUE))
 
+## Not quite optimal, have unnecessary  t(x)  below: _FIXME_
+setMethod("crossprod", signature(x = "matrix", y = "dtrMatrix"),
+	  function(x, y=NULL, boolArith=NA, ...)
+	      if(isTRUE(boolArith)) ## FIXME: very inefficient
+		  crossprod(as(x,"sparseMatrix"), as(y,"sparseMatrix"), boolArith=TRUE)
+	      else
+		  .Call(dtrMatrix_matrix_mm, y, t(x), TRUE, FALSE))
+
 
 ## "dtpMatrix"
 if(FALSE) ## not yet in C
@@ -664,7 +672,7 @@ setMethod("tcrossprod", signature(x = "matrix", y = "missing"),
 		  .Call(dgeMatrix_crossprod, ..2dge(x), TRUE))
 setMethod("tcrossprod", signature(x = "numLike", y = "missing"),
 	  function(x, y=NULL, boolArith=NA, ...)
-	      tcrossprod(as.matrix(as.double(x)), boolArith=boolArith, ...))
+	      tcrossprod(cbind(as.double(x), deparse.level=0L), boolArith=boolArith, ...))
 }# FALSE
 
 setMethod("tcrossprod", signature(x = "ddenseMatrix", y = "missing"),
@@ -686,6 +694,7 @@ setMethod("tcrossprod", signature(x = "dtrMatrix", y = "dtrMatrix"),
 ## setMethod("tcrossprod", signature(x = "dtrMatrix", y = "ddenseMatrix"),
 ##	  function(x, y=NULL, boolArith=NA, ...) .Call(dtrMatrix_matrix_mm, y, x, TRUE, TRUE))
 
+###__ FIXME __ currently goes via geMatrix and loses triangularity !!
 ## setMethod("tcrossprod", signature(x = "dtrMatrix", y = "matrix"),
 ##	  function(x, y=NULL, boolArith=NA, ...) .Call(dtrMatrix_matrix_mm, y, x, TRUE, TRUE))
 
