@@ -1,7 +1,7 @@
 #include <limits.h>
 
-#include "Mutils.h"
 #include <R_ext/Lapack.h>
+#include "Mutils.h"
 
 // La_norm_type() & La_rcond_type()  have been in R_ext/Lapack.h
 //  but have still not been available to package writers ...
@@ -43,33 +43,38 @@ char La_rcond_type(const char *typstr)
 
 double get_double_by_name(SEXP obj, char *nm)
 {
-    SEXP nms = getAttrib(obj, R_NamesSymbol);
+    SEXP nms = PROTECT(getAttrib(obj, R_NamesSymbol));
     int i, len = length(obj);
 
     if ((!isReal(obj)) || (length(obj) > 0 && nms == R_NilValue))
 	error(_("object must be a named, numeric vector"));
     for (i = 0; i < len; i++) {
 	if (!strcmp(nm, CHAR(STRING_ELT(nms, i)))) {
+	    UNPROTECT(1);
 	    return REAL(obj)[i];
 	}
     }
+    UNPROTECT(1);
     return R_NaReal;
 }
 
 SEXP
 set_double_by_name(SEXP obj, double val, char *nm)
 {
-    SEXP nms = getAttrib(obj, R_NamesSymbol);
+    SEXP nms = PROTECT(getAttrib(obj, R_NamesSymbol));
     int i, len = length(obj);
 
     if ((!isReal(obj)) || (length(obj) > 0 && nms == R_NilValue))
 	error(_("object must be a named, numeric vector"));
+    // case 1:  replace existing entry named  <nm>
     for (i = 0; i < len; i++) {
 	if (!strcmp(nm, CHAR(STRING_ELT(nms, i)))) {
 	    REAL(obj)[i] = val;
+	    UNPROTECT(1);
 	    return obj;
 	}
     }
+    // case 2:  no such name --> add new entry with that name at end of vec
     {
 	SEXP nx = PROTECT(allocVector(REALSXP, len + 1)),
 	    nnms = allocVector(STRSXP, len + 1);
@@ -81,7 +86,7 @@ set_double_by_name(SEXP obj, double val, char *nm)
 	}
 	REAL(nx)[len] = val;
 	SET_STRING_ELT(nnms, len, mkChar(nm));
-	UNPROTECT(1);
+	UNPROTECT(2);
 	return nx;
     }
 }
@@ -129,18 +134,18 @@ SEXP get_factors(SEXP obj, char *nm)
 */
 SEXP set_factors(SEXP obj, SEXP val, char *nm)
 {
+    PROTECT(val); /* set_factors(..) may be called as "finalizer" after UNPROTECT()*/
     SEXP fac = GET_SLOT(obj, Matrix_factorSym),
-	nms = getAttrib(fac, R_NamesSymbol);
+	nms = PROTECT(getAttrib(fac, R_NamesSymbol));
     int i, len = length(fac);
 
     if ((!isNewList(fac)) || (length(fac) > 0 && nms == R_NilValue))
 	error(_("'factors' slot must be a named list"));
-    PROTECT(val); /* set_factors(..) may be called as "finalizer" after UNPROTECT()*/
     // if there's already a 'nm' factor, we replace it and return:
     for (i = 0; i < len; i++) {
 	if (!strcmp(nm, CHAR(STRING_ELT(nms, i)))) {
 	    SET_VECTOR_ELT(fac, i, duplicate(val));
-	    UNPROTECT(1);
+	    UNPROTECT(2);
 	    return val;
 	}
     }
@@ -158,7 +163,7 @@ SEXP set_factors(SEXP obj, SEXP val, char *nm)
     SET_VECTOR_ELT(nfac, len, duplicate(val));
     SET_STRING_ELT(nnms, len, mkChar(nm));
     SET_SLOT(obj, Matrix_factorSym, nfac);
-    UNPROTECT(3);
+    UNPROTECT(4);
     return VECTOR_ELT(nfac, len);
 }
 
@@ -174,6 +179,27 @@ SEXP R_set_factors(SEXP obj, SEXP val, SEXP name, SEXP warn)
     }
 }
 
+// R interface for emptying the '@ factors' slot of *function *argument* 'obj' [CARE!]
+SEXP R_empty_factors(SEXP obj, SEXP warn)
+{
+    Rboolean do_warn = asLogical(warn), modified = FALSE;
+    PROTECT(obj);
+    if(R_has_slot(obj, Matrix_factorSym)) {
+	SEXP fac = GET_SLOT(obj, Matrix_factorSym);
+	if (length(fac) > 0) { // if there's a non-empty factor slot, replace it with list()
+	    SEXP fac = PROTECT(allocVector(VECSXP, 0));
+	    SET_SLOT(obj, Matrix_factorSym, fac);
+	    modified = TRUE;
+	    UNPROTECT(1);
+	}
+    } else {
+	if(do_warn) warning(_("Matrix object has no 'factors' slot"));
+    }
+    UNPROTECT(1);
+    return ScalarLogical(modified);
+}
+
+
 #if 0 				/* unused */
 /* useful for all the ..CMatrix classes (and ..R by [0] <-> [1]); but unused */
 SEXP CMatrix_set_Dim(SEXP x, int nrow)
@@ -310,7 +336,7 @@ Rboolean equal_string_vectors(SEXP s1, SEXP s2)
 SEXP dense_nonpacked_validate(SEXP obj)
 {
     int *dims = INTEGER(GET_SLOT(obj, Matrix_DimSym));
-    if ((dims[0] * dims[1]) != length(GET_SLOT(obj, Matrix_xSym)))
+    if ((((double) dims[0]) * dims[1]) != XLENGTH(GET_SLOT(obj, Matrix_xSym)))
 	return mkString(_("length of x slot != prod(Dim)"));
     return ScalarLogical(1);
 }
@@ -318,8 +344,11 @@ SEXP dense_nonpacked_validate(SEXP obj)
 SEXP dim_validate(SEXP Dim, const char* name) {
     if (length(Dim) != 2)
 	return mkString(_("Dim slot must have length 2"));
-    if (TYPEOF(Dim) != INTSXP && TYPEOF(Dim) != REALSXP)
-	return mkString(_("Dim slot is not numeric"));
+    /* if (TYPEOF(Dim) != INTSXP && TYPEOF(Dim) != REALSXP) */
+    /* 	return mkString(_("Dim slot is not numeric")); */
+    if (TYPEOF(Dim) != INTSXP)
+	// TODO?: coerce REALSXP to INTSXP in the "double" case ???
+    	return mkString(_("Dim slot is not integer"));
     int
 	m = INTEGER(Dim)[0],
 	n = INTEGER(Dim)[1];
@@ -336,16 +365,28 @@ SEXP Dim_validate(SEXP obj, SEXP name) {
 			CHAR(STRING_ELT(name, 0)));
 }
 
-// obj must have @Dim and @Dimnames. Assume 'Dim' is already checked.
-SEXP dimNames_validate(SEXP obj)
+/**
+ * Validate dimnames (the @Dim slot typically), assuming
+ * that 'dims' is already checked.
+ *
+ * Utility, called from  dimNames_validate(), but available to other routines.
+ *
+ * @param dmNms an R object should be valid `dimnames`
+ * @param dims  an integer vector of length 2 (must have been checked by caller!).
+
+ * @return a SEXP, either TRUE (= success) or an error message string ("character")
+ */
+SEXP dimNames_validate__(SEXP dmNms, int dims[], const char* obj_name)
 {
-    int *dims = INTEGER(GET_SLOT(obj, Matrix_DimSym));
-    SEXP dmNms = GET_SLOT(obj, Matrix_DimNamesSym);
-    if(!isNewList(dmNms))
-	return mkString(_("Dimnames slot is not a list"));
-    if(length(dmNms) != 2)
-	return mkString(_("Dimnames slot is a list, but not of length 2"));
     char buf[99];
+    if(!isNewList(dmNms)) {
+	sprintf(buf, _("%s is not a list"), obj_name);
+	return mkString(buf);
+    }
+    if(length(dmNms) != 2) {
+	sprintf(buf, _("%s is a list, but not of length 2"), obj_name);
+	return mkString(buf);
+    }
     for(int j=0; j < 2; j++) { // x@Dimnames[j] must be NULL or character(length(x@Dim[j]))
 	if(!isNull(VECTOR_ELT(dmNms, j))) {
 	    if(TYPEOF(VECTOR_ELT(dmNms, j)) != STRSXP) {
@@ -364,6 +405,23 @@ SEXP dimNames_validate(SEXP obj)
 }
 
 
+/**
+ * Check R (Matrix-like) object: must have @Dim and @Dimnames.
+ * Assume 'Dim' is already checked.
+ * (typically used in  S4 validity method)
+ *
+ * @param obj an R object (typically inheriting from `Matrix`)
+ *
+ * @return a SEXP, either TRUE (= success) or an error message string ("character")
+ */
+SEXP dimNames_validate(SEXP obj)
+{
+    return dimNames_validate__(/* dmNms = */ GET_SLOT(obj, Matrix_DimNamesSym),
+			       /* dims =  */ INTEGER(GET_SLOT(obj, Matrix_DimSym)),
+			       "Dimnames slot");
+}
+
+
 
 #define PACKED_TO_FULL(TYPE)						\
 TYPE *packed_to_full_ ## TYPE(TYPE *dest, const TYPE *src,		\
@@ -455,6 +513,8 @@ void l_packed_getDiag(int *dest, SEXP x, int n)
 
 //----------------------------------------------------------------------
 
+/** diag(x) <- D  for   x a  <dspMatrix>  or dppMatrix, ..etc
+ */
 SEXP d_packed_setDiag(double *diag, int l_d, SEXP x, int n)
 {
 #define SET_packed_setDiag				\
@@ -494,7 +554,9 @@ SEXP l_packed_setDiag(int *diag, int l_d, SEXP x, int n)
 #define tr_END_packed_setDiag						\
     if (*diag_P(x) == 'U') { /* uni-triangular */			\
 	/* after setting, typically is not uni-triangular anymore: */	\
-	SET_STRING_ELT(GET_SLOT(ret, Matrix_diagSym), 0, mkChar("N"));	\
+	SEXP ch_N = PROTECT(mkChar("N"));				\
+	SET_STRING_ELT(GET_SLOT(ret, Matrix_diagSym), 0, ch_N);		\
+	UNPROTECT(1);							\
     }									\
     END_packed_setDiag
 
@@ -536,8 +598,11 @@ SEXP d_packed_addDiag(double *diag, int l_d, SEXP x, int n)
 SEXP tr_d_packed_addDiag(double *diag, int l_d, SEXP x, int n)
 {
     SEXP ret = PROTECT(d_packed_addDiag(diag, l_d, x, n));
-    if (*diag_P(x) == 'U') /* uni-triangular */
-	SET_STRING_ELT(GET_SLOT(ret, Matrix_diagSym), 0, mkChar("N"));
+    if (*diag_P(x) == 'U') { /* uni-triangular */
+	SEXP ch_N = PROTECT(mkChar("N"));
+	SET_STRING_ELT(GET_SLOT(ret, Matrix_diagSym), 0, ch_N);
+	UNPROTECT(1);
+    }
     UNPROTECT(1);
     return ret;
 }
@@ -654,15 +719,17 @@ SEXP dup_mMatrix_as_geMatrix(SEXP A)
 	MATRIX_VALID_ndense, /* 5  */
 	""};
     SEXP ans, ad = R_NilValue, an = R_NilValue;	/* -Wall */
-    int sz, ctype = Matrix_check_class_etc(A, valid),
+    R_xlen_t sz;
+    int ctype = R_check_class_etc(A, valid),
 	nprot = 1;
     enum dense_enum M_type = ddense /* -Wall */;
 
     if (ctype > 0) { /* a [nld]denseMatrix or [dl]diMatrix object */
-	ad = GET_SLOT(A, Matrix_DimSym);
-	an = GET_SLOT(A, Matrix_DimNamesSym);
 	M_type = (ctype <= 14) ? ddense :
 	    ((ctype <= 14+6) ? ldense : ndense);
+	/// TODO: Return 'A' unduplicated if ctype indicates "?geMatrix"' -- and *new argument* 'check' is TRUE
+	ad = GET_SLOT(A, Matrix_DimSym);
+	an = GET_SLOT(A, Matrix_DimNamesSym);
     }
     else if (ctype < 0) {	/* not a (recognized) classed matrix */
 
@@ -693,10 +760,9 @@ SEXP dup_mMatrix_as_geMatrix(SEXP A)
 		dd[0] = LENGTH(A);					\
 		dd[1] = 1;						\
 	    }								\
-	    SEXP nms = getAttrib(A, R_NamesSymbol);			\
+	    SEXP nms = PROTECT(getAttrib(A, R_NamesSymbol)); nprot++;	\
 	    if(nms != R_NilValue) {					\
-		an = PROTECT(allocVector(VECSXP, 2));			\
-		nprot++;						\
+		an = PROTECT(allocVector(VECSXP, 2)); nprot++;		\
 	        SET_VECTOR_ELT(an, (transpose_if_vec)? 1 : 0, nms);	\
 		/* not needed: SET_VECTOR_ELT(an, 1, R_NilValue); */    \
 	    } /* else nms = NULL ==> an remains NULL */                 \
@@ -706,14 +772,14 @@ SEXP dup_mMatrix_as_geMatrix(SEXP A)
 	DUP_MMATRIX_NON_CLASS(FALSE);
     }
 
-    ans = PROTECT(NEW_OBJECT(MAKE_CLASS(M_type == ddense ? "dgeMatrix" :
-					(M_type == ldense ? "lgeMatrix" :
-					 "ngeMatrix"))));
-#define DUP_MMATRIX_SET_1					\
-    SET_SLOT(ans, Matrix_DimSym, duplicate(ad));		\
-    SET_SLOT(ans, Matrix_DimNamesSym, (LENGTH(an) == 2) ? 	\
-	     duplicate(an): allocVector(VECSXP, 2));		\
-    sz = INTEGER(ad)[0] * INTEGER(ad)[1]
+    ans = PROTECT(NEW_OBJECT_OF_CLASS(M_type == ddense ? "dgeMatrix" :
+				      (M_type == ldense ? "lgeMatrix" :
+					 "ngeMatrix")));
+#define DUP_MMATRIX_SET_1						\
+    SET_SLOT(ans, Matrix_DimSym, duplicate(ad));			\
+    SET_SLOT(ans, Matrix_DimNamesSym, (!isNull(an) && LENGTH(an) == 2) ? \
+	     duplicate(an): allocVector(VECSXP, 2));			\
+    sz = (R_xlen_t) INTEGER(ad)[0] * INTEGER(ad)[1]
 
     DUP_MMATRIX_SET_1;
 
@@ -817,11 +883,11 @@ SEXP dup_mMatrix_as_geMatrix(SEXP A)
 
 SEXP dup_mMatrix_as_dgeMatrix2(SEXP A, Rboolean tr_if_vec)
 {
-    SEXP ans = PROTECT(NEW_OBJECT(MAKE_CLASS("dgeMatrix"))),
+    SEXP ans = PROTECT(NEW_OBJECT_OF_CLASS("dgeMatrix")),
 	ad = R_NilValue , an = R_NilValue;	/* -Wall */
     static const char *valid[] = {"_NOT_A_CLASS_", MATRIX_VALID_ddense, ""};
-    int ctype = Matrix_check_class_etc(A, valid),
-	nprot = 1, sz;
+    R_xlen_t sz;
+    int ctype = R_check_class_etc(A, valid), nprot = 1;
     double *ansx;
 
     if (ctype > 0) {		/* a ddenseMatrix object */
@@ -831,8 +897,7 @@ SEXP dup_mMatrix_as_dgeMatrix2(SEXP A, Rboolean tr_if_vec)
     else if (ctype < 0) {	/* not a (recognized) classed matrix */
 	if (!isReal(A)) {
 	    if (isInteger(A) || isLogical(A)) {
-		A = PROTECT(coerceVector(A, REALSXP));
-		nprot++;
+		A = PROTECT(coerceVector(A, REALSXP)); nprot++;
 	    } else
 		error(_("invalid class '%s' to dup_mMatrix_as_dgeMatrix"),
 		      class_P(A));
@@ -852,7 +917,7 @@ SEXP dup_mMatrix_as_dgeMatrix(SEXP A) {
 
 SEXP new_dgeMatrix(int nrow, int ncol)
 {
-    SEXP ans = PROTECT(NEW_OBJECT(MAKE_CLASS("dgeMatrix"))),
+    SEXP ans = PROTECT(NEW_OBJECT_OF_CLASS("dgeMatrix")),
 	 ad = PROTECT(allocVector(INTSXP, 2));
 
     INTEGER(ad)[0] = nrow;
@@ -1089,6 +1154,7 @@ SEXP Mmatrix(SEXP args)
 	    }
 	    break;
 	case RAWSXP:
+	    // FIXME:  N may overflow size_t !!
 	    memset(RAW(ans), 0, N);
 	    break;
 	default:
@@ -1117,26 +1183,26 @@ SEXP Mmatrix(SEXP args)
     int *d_a = INTEGER(GET_SLOT(a, Matrix_DimSym)),
 	*d_b = INTEGER(GET_SLOT(b, Matrix_DimSym)),
 	n1 = d_a[0], m = d_a[1],
-	n2 = d_b[0],
-	nprot = 1;
-    SEXP ans,
-	a_x = GET_SLOT(a, Matrix_xSym),
-	b_x = GET_SLOT(b, Matrix_xSym);
+	n2 = d_b[0];
     if(d_b[1] != m)
 	error(_("the number of columns differ in R_rbind2_vector: %d != %d"),
 	      m, d_b[1]);
+    SEXP
+	a_x = GET_SLOT(a, Matrix_xSym),
+	b_x = GET_SLOT(b, Matrix_xSym);
+    int nprot = 1;
     // Care: can have "ddenseMatrix" "ldenseMatrix" or "ndenseMatrix"
     if(TYPEOF(a_x) != TYPEOF(b_x)) { // choose the "common type"
 	// Now know: either LGLSXP or REALSXP. FIXME for iMatrix, zMatrix,..
-	if(TYPEOF(a_x) != REALSXP)
-	    a_x = PROTECT(duplicate(coerceVector(a_x, REALSXP)));
-	else if(TYPEOF(b_x) != REALSXP)
-	    b_x = PROTECT(duplicate(coerceVector(b_x, REALSXP)));
-	nprot++;
+	if(TYPEOF(a_x) != REALSXP) {
+	    a_x = PROTECT(duplicate(coerceVector(a_x, REALSXP))); nprot++;
+	} else if(TYPEOF(b_x) != REALSXP) {
+	    b_x = PROTECT(duplicate(coerceVector(b_x, REALSXP))); nprot++;
+	}
     }
 
-    ans = PROTECT(allocVector(TYPEOF(a_x), m * (n1 + n2)));
-    int i, ii = 0;
+    SEXP ans = PROTECT(allocVector(TYPEOF(a_x), m * (n1 + n2)));
+    int ii = 0;
     switch(TYPEOF(a_x)) {
     case LGLSXP: {
 	int
@@ -1144,11 +1210,10 @@ SEXP Mmatrix(SEXP args)
 	    *ax= LOGICAL(a_x),
 	    *bx= LOGICAL(b_x);
 
-#define COPY_a_AND_b_j						\
-/*  FIXME faster: use Memcpy() : */				\
-	for(int j=0; j < m; j++) {				\
-	    for(i=0; i < n1; i++) r[ii++] = ax[j*n1 + i];	\
-	    for(i=0; i < n2; i++) r[ii++] = bx[j*n2 + i];	\
+#define COPY_a_AND_b_j					\
+	for(int j=0; j < m; j++) {			\
+	    Memcpy(r+ii, ax+ j*n1, n1); ii += n1;	\
+	    Memcpy(r+ii, bx+ j*n2, n2); ii += n2;	\
 	}
 
 	COPY_a_AND_b_j;
@@ -1162,7 +1227,6 @@ SEXP Mmatrix(SEXP args)
 	COPY_a_AND_b_j;
     }
     } // switch
-
     UNPROTECT(nprot);
     return ans;
 }
@@ -1278,15 +1342,14 @@ SEXP symmetric_DimNames(SEXP dn) {
 	if (isNull(VECTOR_ELT(dn,1)))					\
 	    SET_VECTOR_ELT(dn, 1, VECTOR_ELT(dn, 0));			\
 	if(do_nm) { /* names(dimnames(.)): */				\
-	    SEXP nms_dn = getAttrib(dn, R_NamesSymbol);			\
+	    SEXP nms_dn = PROTECT(getAttrib(dn, R_NamesSymbol));	\
 	    if(!R_compute_identical(STRING_ELT(nms_dn, 0),		\
 				    STRING_ELT(nms_dn, 1), 16)) {	\
-		PROTECT(nms_dn);					\
 		int J = LENGTH(STRING_ELT(nms_dn, 0)) == 0; /* 0/1 */   \
 		SET_STRING_ELT(nms_dn, !J, STRING_ELT(nms_dn, J));	\
 		setAttrib(dn, R_NamesSymbol, nms_dn);			\
-		UNPROTECT(1);						\
             }								\
+	    UNPROTECT(1);						\
 	}								\
 	UNPROTECT(1)
 
@@ -1332,3 +1395,15 @@ void SET_DimNames_symm(SEXP dest, SEXP src) {
     return;
 }
 
+/**
+ * A safe NEW_OBJECT(MAKE_CLASS(cls)),  where the caller must protect the
+ * return value of this function
+ *
+ * @param an R character string specifying the name of a known S4 class
+ */
+SEXP NEW_OBJECT_OF_CLASS(const char* cls)
+{
+    SEXP ans = NEW_OBJECT(PROTECT(MAKE_CLASS(cls)));
+    UNPROTECT(1);
+    return ans;
+}
